This directory is intended to contain drop-in customizations for TLP.
See full explanation: https://linrunner.de/tlp/settings

The naming scheme is 00-name.conf, the files are read in lexical (aphabetical)
order.

You may also use /etc/tlp.conf directly, which will override any settings in
this directory.

After making changes, run 'tlp start' to activate them without reboot.
